# -*- coding: utf-8 -*-

#  PycaWM
#  patterns.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys

if sys.version_info < (2, 5):
    raise ImportError('debug module needs at least Python 2.5!')

class MultiBorg(object):
    """A MultiBorg object is a Borg which can be inherited.
    All the instances from a specific class share the same state.

    Warning: if B inherits from A, and A inherits from MultiBorg,
    the B instances do NOT share state with A!
    """

    _shared_state = dict()
    def __new__(cls, *args, **kwargs):
        obj = super(MultiBorg, cls).__new__(cls, *args, **kwargs)
        if cls in cls._shared_state:
            obj.__dict__ = cls._shared_state[cls]
        else:
            obj.__dict__ = dict()
            cls._shared_state[cls] = obj.__dict__
        return obj

class Null(object):
    """Null objects always and reliably "do nothing."
    Slightly modified and extended from the Python Cookbook, 2nd edition.
    """

    def __new__(cls, *args, **kwargs):
        if '_inst' not in vars(cls):
            cls._inst = super(Null, cls).__new__(cls, *args, **kwargs)
        # same instance for all
        return cls._inst

    def __init__(self, *args, **kwargs):
        pass

    def __call__(self, *args, **kwargs):
        return self

    def __repr__(self):
        return "Null()"

    def __nonzero__(self):
        return False

    def __len__(self):
        return 0

    def __iter__(self):
        return iter(())

    __getattr__ = __call__
    __setattr__ = __call__
    __delattr__ = __call__
    __getitem__ = __call__
    __setitem__ = __call__
    __delitem__ = __call__

class DictWrapper(object):
    # innocently stolen (and modified a bit) from the python Xlib sources
    def __init__(self, dict_=None):
        if dict_ is None:
            dict_ = dict()
        self.__dict__['_data'] = dict_

    def __getattr__(self, attr):
        if self._data:
            if attr in self._data:
                return self._data[attr]
            else:
                raise AttributeError(attr)
        else:
            raise AttributeError(attr)

    def __getitem__(self, key):
        return self._data[key]

    def __setitem__(self, key, value):
        self._data[key] = value

    def __delitem__(self, key):
        del self._data[key]

    def __setattr__(self, key, value):
        self._data[key] = value

    def __delattr__(self, key):
        del self._data[key]

    def __str__(self):
        return str(self._data)

    def __repr__(self):
        return '%s(%s)' % (self.__class__, repr(self._data))

    def __cmp__(self, other):
        if isinstance(other, DictWrapper):
            return cmp(self._data, other._data)
        else:
            return cmp(self._data, other)
